-module(n2o_sample_sup).

-behaviour(supervisor).

-include_lib("kvs/include/user.hrl").


-export([
  start_link/0,
  init/1
]).


start_link() ->
  supervisor:start_link({local, ?MODULE}, ?MODULE,[]).


-define(USERS, [ #user{id = "maxim",  email = "maxim@synrc.com"},
                 #user{id = "doxtop", email = "doxtop@synrc.com"},
                 #user{id = "roman",  email = "roman@github.com"} ]).


init([]) ->
  users:init(),
  users:populate(?USERS),
  kvs:join(),
  
  Params = #{host => application:get_env(n4u, pgs_host, "localhost"), %% gets settings from sys.config
    port => application:get_env(n4u, pgs_port, 6432),                 %% and start postgresql connections pool
    username => application:get_env(n4u, pgs_user, "user"),
    password => application:get_env(n4u, pgs_pass, "pass"),
    database => application:get_env(n4u, pgs_db, "test")},
  
  case epgsql_pool:start(my_main_pool, 10, 50, Params) of
    {ok, _} ->
      io:format("~p~n", ["pg_pool start !!"]),
      ok;
    Err ->
      io:format("Pool start err: ~p~n~p~n", ["err db connect", Err]),
      err
  end,
  
  {ok, {{one_for_one, 10, 10}, [spec()]}}.


spec()   -> ranch:child_spec(http, 100, ranch_tcp, port(), cowboy_protocol, env()).
env()    -> [ { env, [ { dispatch, points() } ] } ].
static() ->   { dir, "apps/n2o_sample/priv/static", mime() }.
static2() ->  { dir, "apps/n2o_sample/priv/js", mime() }.
%n4u()    ->   { dir, "deps/n4u/priv",           mime() }.
mime()   -> [ { mimetypes, cow_mimetypes, all   } ].
port()   -> [ { port, wf:config(n4u, port, 8000)  } ].
points() -> cowboy_router:compile([{'_', [
    {"/static/[...]",       n2o_static,    static()}, %% static without nginx by cowboy  % todo : clean - mv to stc folder
    {"/js/[...]",           n2o_static,   static2()}, %% static without nginx by cowboy
    {"/css/[...]",          n2o_static,    static()}, %% static without nginx by cowboy
    {"/img/[...]",          n2o_static,    static()}, %% static without nginx by cowboy
%    {"/n4u/[...]",          n2o_static,    n4u()},    %% static without nginx by cowboy
    {"/multipart/[...]",    n2o_multipart, []},
    {"/rest/:resource",     rest_cowboy,   []},
    {"/rest/:resource/:id", rest_cowboy,   []},
    {"/ws/[...]",           n2o_stream,    []},
    {'_',                   n2o_cowboy,    []}
  ]}]).

